#include "konquestwindow.h"

void KonquestWindow::performeClose()
{
    if(communicationLine)
        disconnect(communicationLine,SIGNAL(disconnected()),this,
                   SLOT(disconnectionClose()));
    if(isHost)
    {
        for(int i = 0 ; i < connectedPlayers.count(); ++i)
            disconnect(connectedPlayers.value(i),SIGNAL(disconnected()),this,
                       SLOT(disconnectionClose()));
    }
    chatLog->close();
    this->close();
}

void KonquestWindow::hideShowChatLog()
{
    if(chatLog->isHidden())
    {
        chatLog->show();
        chatLog->move(this->pos().x() + 600, this->pos().y() + 100);
    }
    else
        chatLog->hide();
}

void KonquestWindow::sendTextMsg(QString msg)
{
    if(isHost)
    {
        QString message = tr("%1: %2").arg(nick).arg(msg);
        chatLog->addLogMsg(message,true);
        for(int i = 0; i < connectedPlayers.count(); ++i)
            connectedPlayers.at(i)->write(message.toAscii());
    }
    else
    {
        if(communicationLine)
        {
            QString message = tr("%1: %2").arg(nick).arg(msg);
            communicationLine->write(message.toAscii());
        }
    }
}

void KonquestWindow::mousePressEvent(QMouseEvent* event)
{
    x = event->globalPos().x();
    y = event->globalPos().y();
}

void KonquestWindow::mouseMoveEvent(QMouseEvent* event)
{
    int dx = event->globalPos().x() - x;
    int dy = event->globalPos().y() - y;
    x = event->globalPos().x();
    y = event->globalPos().y();
    this->move(this->pos().x()+dx,this->pos().y()+dy);
    if(!chatLog->isHidden())
        chatLog->move(this->pos().x() + 600, this->pos().y() + 100);
}

void KonquestWindow::keyReleaseEvent(QKeyEvent* e)
{
    if(e->key() == Qt::Key_Escape)
    {
        if(gameSrcPlanet != -1)
        {
            if(planetItems.keys().contains(gameSrcPlanet))
                planetItems.value(gameSrcPlanet)->unMark();
            gameSrcPlanet = -1;
        }
    }
}

void KonquestWindow::planetClicked(int id)
{
    if(planetDataMap.keys().contains(id))
    {
        if(gameSrcPlanet == -1)
        {
            if(planetDataMap.value(id).ownerId == myPlayerId)
            {
                if(currentPlayerTurn == myPlayerId)
                {
                    gameSrcPlanet = id;
                    planetItems.value(gameSrcPlanet)->mark();
                }

                else
                    QMessageBox::warning(this,tr("Not Your Turn"),
                                         tr("Please wait for your turn!"));
            }
        }
        else
        {
            if(planetDataMap.keys().contains(id) && id != gameSrcPlanet)
            {
                int shipCount = QInputDialog::getInt(this,tr("Send Ships"),
                                                     tr("Number of ships to send:"),
                                                     0,0,
                                                     planetDataMap.
                                                     value(gameSrcPlanet).
                                                     shipCount,
                                                     1);
                if(shipCount > 0)
                {
                    if(isHost)
                    {
                        if(makeMove(gameSrcPlanet,id,shipCount))
                        {
                            QString msg = tr("[move];%1;%2;%3;")
                                          .arg(gameSrcPlanet)
                                          .arg(id)
                                          .arg(shipCount);
                            for(int i = 0; i < connectedPlayers.count(); ++i)
                                connectedPlayers.value(i)->write(msg.toAscii());
                        }
                    }
                    else
                    {
                        if(communicationLine)
                        {
                            QString msg = tr("[move];%1;%2;%3;")
                                          .arg(gameSrcPlanet)
                                          .arg(id)
                                          .arg(shipCount);
                            communicationLine->write(msg.toAscii());
                        }
                    }
                }
                if(planetItems.keys().contains(gameSrcPlanet))
                    planetItems.value(gameSrcPlanet)->unMark();
                gameSrcPlanet = -1;
            }
        }
    }
}

void KonquestWindow::planetTouched(int id,int x,int y)
{
    if(planetDataMap.keys().contains(id))
    {
        QString ownerName = "Pirates";
        if(planetDataMap.value(id).ownerId != -1
           && planetDataMap.value(id).ownerId < playerNames.count())
            ownerName = playerNames.value(planetDataMap.value(id).ownerId);

        infoDisplay->setData(planetDataMap.value(id).planetName,
                             ownerName,
                             planetDataMap.value(id).production,
                             planetDataMap.value(id).shipCount,
                             planetDataMap.value(id).defensBonus);
        int newX = x+20;
        int newY = y+20;

        if((newX + infoDisplay->getWidth()) > 410)
            newX = 410 - infoDisplay->getWidth();
        if((newY + 52) > 410)
            newY = 358;

        infoDisplay->setPos(newX,newY);
        infoDisplay->show();
    }
}

void KonquestWindow::planetUntouched()
{
    infoDisplay->hide();
}

//---------------------------------Host-----------------------------------------
void KonquestWindow::hostGame()
{
    if(communicationLine)
    {
        QMessageBox::warning(this,tr("Can not Host"),
                             tr("You cant host a game when your connected to a remote host!"));
        return;
    }
    if(isHost)
    {
        QMessageBox::warning(this,tr("Can not Host"),
                             tr("You are alrady hosting a game!"));
        return;
    }
    ipEdit->setEnabled(false);
    playerNames.clear();
    nick = "Host";
    playerNames[0] = "Host";
    nameEdit->setEnabled(true);
    nameEdit->setText(nick);
    isHost = true;
    server = new ServerThread();
    connect(server,SIGNAL(playerConnected(QTcpSocket*)),this,
            SLOT(newPlayerJoined(QTcpSocket*)),Qt::QueuedConnection);
    connect(server,SIGNAL(serverAddress(QString)),this,
            SLOT(readServerAddress(QString)),Qt::QueuedConnection);
    connect(server,SIGNAL(serverCantListen()),this,SLOT(serverListenError()));
    server->run();
    myPlayerId = 0;
}

void KonquestWindow::readServerAddress(QString address)
{
    ipEdit->setText(address);
}

void KonquestWindow::startGame()
{
    if(isHost)
    {
        if(playerNames.count() >= 2)
        {
            server->gameStart();
            makePlanets();
            updateOwnPlanetShipCount();
            sendPlayerList();
            sendPlanets();
            currentPlayerTurn = 0;
            QString msg = tr("[turnNo];%1;").arg(currentPlayerTurn);
            for(int i = 0; i < connectedPlayers.count(); ++i)
                connectedPlayers.value(i)->write(msg.toAscii());
            if(currentPlayerTurn == myPlayerId)
                tLabel->setPixmap(QPixmap(":/eyecandy/green.png"));
            else
                tLabel->setPixmap(QPixmap(":/eyecandy/red.png"));
            dayCounter = 1;
        }
        else
            QMessageBox::warning(this,tr("Can not Start"),
                                 tr("You need at least 2 palyers!"));
    }
    else
        QMessageBox::warning(this,tr("Can not Start"),
                             tr("You are not hosting a game!"));
}

void KonquestWindow::newPlayerJoined(QTcpSocket* connection)
{
    if(connectedPlayers.count() < 3)
    {
        QString name(tr("Player %1").arg(connectedPlayers.count()+1));
        connectedPlayers << connection;
        playerNames[connectedPlayers.count()] = name;
        connection->write(tr("[nick];%1;[playerNo];%2;").
                          arg(name)
                          .arg(connectedPlayers.count())
                          .toAscii());
        connect(connection,SIGNAL(readyRead()),this,
                SLOT(massageFromPlayer()),Qt::QueuedConnection);
        connect(connection,SIGNAL(disconnected()),this,
                SLOT(disconnectionClose()));
    }
    else
    {
        connection->write("Player List full");
        connection->disconnectFromHost();
    }
}

void KonquestWindow::massageFromPlayer()
{
    if(isHost)
    {
        QTcpSocket* sender = qobject_cast<QTcpSocket*>(QObject::sender());
        if(sender)
        {
            QString msg = sender->readAll();
            if(msg.startsWith("["))//command
            {
                QStringList commandList = msg.split(";");
                while(commandList.count()>0)
                {
                    QString command = commandList.takeFirst();
                    if(command == "[move]")
                    {
                        if(commandList.count() >= 3)
                        {
                            int scrPlanet = commandList.takeFirst().toInt();
                            int destPlanet = commandList.takeFirst().toInt();
                            int shipCount = commandList.takeFirst().toInt();
                            if(makeMove(scrPlanet,destPlanet,shipCount))
                            {
                                QString msg = tr("[move];%1;%2;%3;")
                                              .arg(scrPlanet)
                                              .arg(destPlanet)
                                              .arg(shipCount);
                                for(int i = 0; i < connectedPlayers.count(); ++i)
                                    connectedPlayers.value(i)->write(msg.toAscii());
                            }
                        }
                    }
                    else if(command == "[nextTurn]")
                    {
                        if(currentPlayerTurn == playerNames.count()-1)
                        {
                            currentPlayerTurn = 0;
                            for(int i = 0; i < connectedPlayers.count(); ++i)
                                connectedPlayers.value(i)->write("[dayEnd];");
                            nextDay();
                            QString msg(tr("----End of Day %1----").arg(dayCounter));
                            chatLog->addLogMsg(msg);
                            for(int i = 0; i < connectedPlayers.count(); ++i)
                                connectedPlayers.value(i)->write(msg.toAscii());
                            ++dayCounter;
                        }
                        else
                        {
                            ++currentPlayerTurn;
                            QString msg = tr("[turnNo];%1;").arg(currentPlayerTurn);
                            for(int i = 0; i < connectedPlayers.count(); ++i)
                                connectedPlayers.value(i)->write(msg.toAscii());
                        }
                        if(currentPlayerTurn == myPlayerId)
                            tLabel->setPixmap(QPixmap(":/eyecandy/green.png"));
                        else
                            tLabel->setPixmap(QPixmap(":/eyecandy/red.png"));
                    }
                    else if(command == "[rename]")
                    {
                        if(commandList.count() >= 2)
                        {
                            QString wantedName = commandList.takeFirst();
                            int playerId = commandList.takeFirst().toInt();
                            if(!playerNames.values().contains(wantedName))
                            {
                                QString oldName = playerNames.value(playerId);
                                playerNames[playerId] = wantedName;
                                sendPlayerList();
                                QString msg = tr("%1 -> %2")
                                              .arg(oldName)
                                              .arg(wantedName);
                                for(int i = 0; i < connectedPlayers.count(); ++i)
                                    connectedPlayers.at(i)->write(msg.toAscii());
                                chatLog->addLogMsg(msg);
                            }
                        }
                    }
                    else if(!command.startsWith("["))
                    {
                        chatLog->addLogMsg(command,true);
                        for(int i = 0; i < connectedPlayers.count(); ++i)
                            connectedPlayers.value(i)->write(command.toAscii());
                    }
                }
            }
            else//text message
            {
                chatLog->addLogMsg(msg,true);
                for(int i = 0; i < connectedPlayers.count(); ++i)
                    connectedPlayers.value(i)->write(msg.toAscii());
            }

        }
    }
}

void KonquestWindow::sendPlayerList()
{
    for(int i = 0; i < connectedPlayers.count(); ++i)
    {
        QTcpSocket* comLine = connectedPlayers.value(i);
        for(int j = 0; j < playerNames.keys().count(); ++j)
        {
            int id = playerNames.keys().value(j);
            QString playerName = playerNames[id];
            QString msg = tr("[palyerList];%1;%2;")
                          .arg(id)
                          .arg(playerName);
            comLine->write(msg.toAscii());
        }
    }
}

void KonquestWindow::sendPlanets()
{
    for(int i = 0; i < connectedPlayers.count(); ++i)
    {
        QTcpSocket* comLine = connectedPlayers.value(i);
        for(int j = 0; j < planetDataMap.keys().count(); ++j)
        {
            int key = planetDataMap.keys().value(j);
            QString planetName = planetDataMap[key].planetName;
            int production = planetDataMap[key].production;
            int shipCount = planetDataMap[key].shipCount;
            int ownerId = planetDataMap[key].ownerId;
            float defensBonus = planetDataMap[key].defensBonus;
            int x = planetDataMap[key].pos.x;
            int y = planetDataMap[key].pos.y;
            QString msg = tr("[addPlanet];%1;%2;%3;%4;%5;%6;%7;%8;")
                          .arg(planetName)
                          .arg(production)
                          .arg(shipCount)
                          .arg(ownerId)
                          .arg(defensBonus)
                          .arg(x)
                          .arg(y)
                          .arg(key);
            comLine->write(msg.toAscii());
        }
    }
}

void KonquestWindow::serverListenError()
{
    QMessageBox::warning(this,tr("Can not Host"),
                             tr("The server listening port is closed!"));
    isHost = false;
    delete server;
    server = NULL;
    ipEdit->setEnabled(true);
}
//------------------------------------------------------------------------------

//----------------------------------Guest---------------------------------------
void KonquestWindow::connectToGame()
{
    if(!isHost)
    {
        if(!communicationLine)
        {
            QHostAddress address;
            if(address.setAddress(ipEdit->text()))
            {
                communicationLine = new QTcpSocket(this);
                communicationLine->connectToHost(address,6666);
                connect(communicationLine,SIGNAL(readyRead()),this,
                        SLOT(incommingMsg()));
                connect(communicationLine,SIGNAL(disconnected()),this,
                        SLOT(disconnectionClose()));
                ipEdit->setEnabled(false);
            }
            else
                QMessageBox::warning(this,tr("Bad Ip"),tr("Wrang IP address"));
        }
    }
}

void KonquestWindow::incommingMsg()
{
    if(!isHost)
    {
        QString msg = communicationLine->readAll();
        if(msg.startsWith("["))
        {
            QStringList commandList = msg.split(";");
            while(commandList.count()>0)
            {
                QString command = commandList.takeFirst();
                if(command == "[nick]")
                {
                    if(commandList.count() > 0)
                    {
                        nick = commandList.takeFirst();
                        nameEdit->setEnabled(true);
                        nameEdit->setText(nick);
                    }
                }
                else if(command == "[playerNo]")
                {
                    if(commandList.count() > 0)
                    {
                        myPlayerId = commandList.takeFirst().toInt();
                        makeColorLabel();
                    }
                }
                else if(command == "[addPlanet]")
                {
                    if(commandList.count() >= 8)
                    {
                        QString planetName = commandList.takeFirst();
                        int production = commandList.takeFirst().toInt();
                        int shipCount = commandList.takeFirst().toInt();
                        int ownerId = commandList.takeFirst().toInt();
                        float defensBonus = commandList.takeFirst().toFloat();
                        int x = commandList.takeFirst().toInt();
                        int y = commandList.takeFirst().toInt();
                        int id = commandList.takeFirst().toInt();

                        PlanetData data(planetName,production,shipCount,ownerId,
                                        defensBonus,PlanetPos(x,y));
                        planetDataMap.insert(id,data);
                        //qsrand(QDateTime::currentDateTime().time().msec());
                        PlanetItem* item = new PlanetItem(QPixmap(
                                tr(":/eyecandy/planet%1.png").arg(qrand()%21)),id);
                        ui.gameView->scene()->addItem(item);
                        item->setPos(41 * x,41 * y);

                        connect(item,SIGNAL(clicked(int)),this,SLOT(planetClicked(int)));
                        connect(item,SIGNAL(touched(int,int,int)),
                                this,SLOT(planetTouched(int,int,int)));
                        connect(item,SIGNAL(unTouched()),this,SLOT(planetUntouched()));

                        planetItems.insert(id,item);
                        if(ownerId != -1)
                        {
                            item->setOwnerColor(colorList.at(ownerId));
                        }
                        updateOwnPlanetShipCount();
                    }
                }
                else if(command == "[palyerList]")
                {
                    if(commandList.count() >=2)
                    {
                        int id = commandList.takeFirst().toInt();
                        QString playerName = commandList.takeFirst();
                        playerNames[id] = playerName;
                        if(id == myPlayerId)
                        {
                            nick = playerName;
                            nameEdit->setText(nick);
                        }
                    }
                }
                else if(command == "[move]")
                {
                    if(commandList.count() >= 3)
                    {
                        int scrPlanet = commandList.takeFirst().toInt();
                        int destPlanet = commandList.takeFirst().toInt();
                        int shipCount = commandList.takeFirst().toInt();
                        makeMove(scrPlanet,destPlanet,shipCount);
                    }
                }
                else if(command == "[turnNo]")
                {
                    currentPlayerTurn = commandList.takeFirst().toInt();
                    if(currentPlayerTurn == myPlayerId)
                        tLabel->setPixmap(QPixmap(":/eyecandy/green.png"));
                    else
                        tLabel->setPixmap(QPixmap(":/eyecandy/red.png"));
                }
                else if(command == "[dayEnd]")
                {
                    currentPlayerTurn = 0;
                    nextDay();
                    if(currentPlayerTurn == myPlayerId)
                        tLabel->setPixmap(QPixmap(":/eyecandy/green.png"));
                    else
                        tLabel->setPixmap(QPixmap(":/eyecandy/red.png"));
                }
                else if(!command.startsWith("["))
                {
                    chatLog->addLogMsg(command,true);
                }
            }
        }
        else
        {
            chatLog->addLogMsg(msg,true);
        }
    }
}
//------------------------------------------------------------------------------

void KonquestWindow::nextTurn()
{
    if(isHost)
    {
        if(currentPlayerTurn == myPlayerId)
        {
            if(currentPlayerTurn == playerNames.count()-1)
            {
                currentPlayerTurn = 0;
                for(int i = 0; i < connectedPlayers.count(); ++i)
                    connectedPlayers.value(i)->write("[dayEnd];");
                nextDay();
                QString msg(tr("----End of Day %1----").arg(dayCounter));
                chatLog->addLogMsg(msg);
                for(int i = 0; i < connectedPlayers.count(); ++i)
                    connectedPlayers.value(i)->write(msg.toAscii());
                ++dayCounter;
            }
            else
            {
                ++currentPlayerTurn;
                QString msg = tr("[turnNo];%1;").arg(currentPlayerTurn);
                for(int i = 0; i < connectedPlayers.count(); ++i)
                    connectedPlayers.value(i)->write(msg.toAscii());
            }
            if(currentPlayerTurn == myPlayerId)
                tLabel->setPixmap(QPixmap(":/eyecandy/green.png"));
            else
                tLabel->setPixmap(QPixmap(":/eyecandy/red.png"));
        }
        else
            QMessageBox::warning(this,tr("Cant end turn")
                                 ,tr("You cant end a turn that is not yours."));
    }
    else
    {
        if(currentPlayerTurn == myPlayerId)
        {
            if(communicationLine)
                communicationLine->write("[nextTurn]");
        }
        else
            QMessageBox::warning(this,tr("Cant end turn")
                                 ,tr("You cant end a turn that is not yours."));
    }
    if(planetItems.keys().contains(gameSrcPlanet))
        planetItems.value(gameSrcPlanet)->unMark();
    gameSrcPlanet = -1;
}

void KonquestWindow::nextDay()
{
    for(int i = 0; i < planetDataMap.keys().count(); ++i)
    {
        int key = planetDataMap.keys().value(i);

        if(planetDataMap.value(key).ownerId != -1)
        {
            planetDataMap[key].shipCount += planetDataMap.value(key).production;
        }
        else//pirates pirates production = production * 0.25f
        {
            planetDataMap[key].shipCount += static_cast<int>(
                    planetDataMap.value(key).production * 0.25f);
        }

        QList<Fleet> newFleetList;
        for(int j = 0; j < planetDataMap.value(key).inboundFleets.count(); ++j)
        {
            --planetDataMap[key].inboundFleets[j].roundsToArrival;
            if(planetDataMap.value(key).inboundFleets.at(j).roundsToArrival == 0)
            {
                if(planetDataMap.value(key).ownerId ==
                   planetDataMap.value(key).inboundFleets.at(j).ownerId)//friend
                {
                    planetDataMap[key].shipCount +=
                            planetDataMap.value(key).inboundFleets.at(j).shipCount;
                    chatLog->addLogMsg(tr("%1 reinforcements arrived at %2")
                                       .arg(planetDataMap.value(key).
                                            inboundFleets.at(j).shipCount)
                                       .arg(planetDataMap.value(key)
                                            .planetName));
                }
                else //fo
                {
                    int atackerKillCount = planetDataMap.value(key).
                                           inboundFleets.at(j).shipCount;

                    float defBouns = planetDataMap.value(key).defensBonus;
                    if(planetDataMap.value(key).ownerId == -1) //pirates def bonus 0.2
                        defBouns = 0.2f;

                    int defenderKillCount = planetDataMap.value(key).shipCount +
                                            static_cast<int>(
                                                    planetDataMap.value(key)
                                                    .shipCount * defBouns);

                    if(defenderKillCount >= atackerKillCount) //defens succesful
                    {
                        planetDataMap[key].shipCount -=
                                static_cast<int>(atackerKillCount * 0.5f);
                        chatLog->addLogMsg(tr("%1 stood against an attack from %2")
                                           .arg(planetDataMap.value(key).planetName)
                                           .arg(playerNames.value(
                                                   planetDataMap.value(key).
                                                   inboundFleets.at(j).ownerId)));
                    }
                    else //atackers win
                    {
                        planetDataMap[key].shipCount = atackerKillCount -
                                                       static_cast<int>(
                                                               defenderKillCount * 0.5f);
                        planetDataMap[key].ownerId =
                                planetDataMap.value(key).inboundFleets.at(j).ownerId;

                        planetItems.value(key)->setOwnerColor(colorList.at(
                                planetDataMap.value(key).ownerId));
                        chatLog->addLogMsg(tr("%1 has fallen to %2")
                                           .arg(planetDataMap.value(key).planetName)
                                           .arg(playerNames.value(
                                                   planetDataMap.value(key).
                                                   inboundFleets.at(j).ownerId)));
                    }
                }
            }
            else
                newFleetList << planetDataMap.value(key).inboundFleets.at(j);
        }

        planetDataMap[key].inboundFleets = newFleetList;
    }
    updateOwnPlanetShipCount();
}

bool KonquestWindow::makeMove(int srcPlanet,int dstPlanet,int shipCount)
{
    bool resault = false;
    if(planetDataMap.keys().contains(srcPlanet) &&
       planetDataMap.keys().contains(dstPlanet))
    {
        if(planetDataMap.value(srcPlanet).shipCount >= shipCount)
        {
            int ownerId = planetDataMap.value(srcPlanet).ownerId;

            unsigned int dx = planetDataMap.value(srcPlanet).pos.x -
                              planetDataMap.value(dstPlanet).pos.x;
            unsigned int dy = planetDataMap.value(srcPlanet).pos.y -
                              planetDataMap.value(dstPlanet).pos.y;
            int travelTime = static_cast<int>(
                    sqrt(static_cast<double>(dx*dx+dy*dy)) + 0.6);

            if(ownerId == myPlayerId)
            {
                QString msg;
                if(travelTime == 1)
                    msg = tr("A fleet of %1 ships\nfrom %2 will arrive at %3\nin %4 day")
                              .arg(shipCount)
                              .arg(planetDataMap.value(srcPlanet).planetName)
                              .arg(planetDataMap.value(dstPlanet).planetName)
                              .arg(travelTime);
                else
                    msg = tr("A fleet of %1 ships\nfrom %2 will arrive at %3\nin %4 days")
                              .arg(shipCount)
                              .arg(planetDataMap.value(srcPlanet).planetName)
                              .arg(planetDataMap.value(dstPlanet).planetName)
                              .arg(travelTime);
                chatLog->addLogMsg(tr("-------------MOVE--------------"));
                chatLog->addLogMsg(msg);
                chatLog->addLogMsg(tr("-------------------------------"));
            }

            Fleet newFleet(shipCount,ownerId,travelTime);

            planetDataMap[srcPlanet].shipCount -= shipCount;

            planetDataMap[dstPlanet].addFleet(newFleet);

            resault = true;
            updateOwnPlanetShipCount();
        }
    }
    return resault;
}

void KonquestWindow::disconnectionClose()
{
    QMessageBox::warning(this,tr("Connection lost"),
                         tr("A connection problem has been encountered."));
    performeClose();
}

void KonquestWindow::changeName()
{
    if(isHost)
    {
        if(nameEdit->text().trimmed() != "")
        {
            QString oldName = nick;
            nick = nameEdit->text().trimmed();
            playerNames[myPlayerId] = nick;
            sendPlayerList();
            QString msg = tr("%1 -> %2")
                          .arg(oldName)
                          .arg(nick);
            for(int i = 0; i < connectedPlayers.count(); ++i)
                connectedPlayers.at(i)->write(msg.toAscii());
            chatLog->addLogMsg(msg);
            ui.gameView->setFocus();
        }
        else
            nameEdit->setText(nick);
    }
    else
    {
        if(communicationLine)
        {
            if(nameEdit->text().trimmed() != "")
            {
                QString msg = tr("[rename];%1;%2;")
                              .arg(nameEdit->text().trimmed())
                              .arg(myPlayerId);
                communicationLine->write(msg.toAscii());
                ui.gameView->setFocus();
            }
            nameEdit->setText(nick);
        }
    }
}

void KonquestWindow::makeGrid()
{
    QPen pen(QBrush(QColor(255,255,255,55)),1.0f,Qt::DashLine,
             Qt::RoundCap,Qt::RoundJoin);
    for(int i = 1; i < 10; ++i)
    {
        QGraphicsLineItem* line = new QGraphicsLineItem(i*41,0,i*41,600);
        line->setPen(pen);
        line->setZValue(-1);
        ui.gameView->scene()->addItem(line);
        gridVector<<line;
    }
    for(int i = 1; i < 10; ++i)
    {
        QGraphicsLineItem* line = new QGraphicsLineItem(0,i*41,600,i*41);
        line->setPen(pen);
        line->setZValue(-1);
        ui.gameView->scene()->addItem(line);
        gridVector<<line;
    }
    hideShowGrid();
}

void KonquestWindow::hideShowGrid()
{
    if(gridVisable)
        for(int i = 0; i<gridVector.count(); ++i)
            gridVector.at(i)->hide();
    else
        for(int i = 0; i<gridVector.count(); ++i)
            gridVector.at(i)->show();
    gridVisable = !gridVisable;
}

void KonquestWindow::makePlanets()
{
    //making planets host only
    if(isHost)
    {
        QList<PlanetPos> usedPosList;
        for(int i = 0; i < 36; ++i)
        {
            qsrand(QDateTime::currentDateTime().time().msec()*i+1);
            PlanetItem* item = new PlanetItem(QPixmap(
                    tr(":/eyecandy/planet%1.png").arg(qrand()%21)),i);
            ui.gameView->scene()->addItem(item);
            PlanetPos pos(static_cast<unsigned int>(qrand()%10),
                          static_cast<unsigned int>(qrand()%10));
            while(usedPosList.contains(pos))
                pos = PlanetPos(static_cast<unsigned int>(qrand()%10),
                                static_cast<unsigned int>(qrand()%10));
            usedPosList.append(pos);
            item->setPos(pos.x * 41, pos.y * 41);
            PlanetData data(tr("Planet %1").arg(i),
                            static_cast<unsigned int>((qrand()%12)+6),//planet production
                            10,//planet starting ships
                            -1,//palnet owner -1 pirates
                            0.1f * static_cast<unsigned int>((qrand()%4)+3),//defens bonus
                            pos);
            planetDataMap.insert(i,data);
            planetItems.insert(i,item);
            connect(item,SIGNAL(clicked(int)),this,SLOT(planetClicked(int)));
            connect(item,SIGNAL(touched(int,int,int)),
                    this,SLOT(planetTouched(int,int,int)));
            connect(item,SIGNAL(unTouched()),this,SLOT(planetUntouched()));
        }
        for(int i = 0; i < connectedPlayers.count() + 1; ++i)
        {
            planetDataMap[i*5].ownerId = i;
            planetItems.value(i*5)->setOwnerColor(colorList.at(i));
        }
    }
}

void KonquestWindow::makeColorLabel()
{
    QPixmap drowing(20,20);
    QPainter painter(&drowing);
    painter.setBrush(QBrush(colorList.at(myPlayerId)));
    painter.setPen(Qt::transparent);
    painter.drawRect(0,0,20,20);
    colorLabel->setPixmap(drowing);
}

void KonquestWindow::updateOwnPlanetShipCount()
{
    if(isHost || communicationLine)
    {
        for(int i = 0; i < planetDataMap.keys().count(); ++i)
        {
            int key = planetDataMap.keys().at(i);
            if(planetDataMap.value(key).ownerId == myPlayerId)
            {
                planetItems[key]->setShipCount(
                        planetDataMap.value(key).shipCount);
                planetItems.value(key)->showShipCount();
            }
            else
                planetItems.value(key)->hideShipCount();
        }
    }
}

void KonquestWindow::makeButtons()
{
    tLabel = new QLabel();
    tLabel->setPixmap(QPixmap(":/eyecandy/red.png"));
    tLabel->setMaximumHeight(38);
    tLabel->setMaximumWidth(38);
    ui.topLayout->addWidget(tLabel);

    ipEdit = new QLineEdit();
    ipEdit->setMaximumWidth(100);
    ipEdit->setInputMask("000.000.000.000;_");
    ui.topLayout->addWidget(ipEdit);
    connect(ipEdit,SIGNAL(returnPressed()),this,SLOT(connectToGame()));

    colorLabel = new QLabel();
    colorLabel->setMaximumWidth(20);
    colorLabel->setMaximumHeight(20);
    ui.topLayout->addWidget(colorLabel);

    nameEdit = new QLineEdit();
    nameEdit->setMaximumWidth(100);
    ui.topLayout->addWidget(nameEdit);
    connect(nameEdit,SIGNAL(returnPressed()),this,SLOT(changeName()));
    nameEdit->setEnabled(false);

    RoundButton* close = new RoundButton(QPixmap(":/eyecandy/close.png"));
    close->setToolTip(tr("Close"));
    connect(close,SIGNAL(clicked()),this,SLOT(performeClose()));
    ui.topLayout->addWidget(close);

    RoundButton* hideShowGrid = new RoundButton(QPixmap(":/eyecandy/grid.png"));
    hideShowGrid->setToolTip(tr("Hide Show Grid"));
    connect(hideShowGrid,SIGNAL(clicked()),this,SLOT(hideShowGrid()));
    ui.bottomLayout->addWidget(hideShowGrid);

    RoundButton* hideShowChat = new RoundButton(QPixmap(":/eyecandy/chat.png"));
    hideShowChat->setToolTip(tr("Hide Show Chat/Log"));
    connect(hideShowChat,SIGNAL(clicked()),this,SLOT(hideShowChatLog()));
    ui.rightLayout->addWidget(hideShowChat);

    RoundButton* sth = new RoundButton(QPixmap(":/eyecandy/sth.png"));
    sth->setToolTip(tr("Host Game"));
    connect(sth,SIGNAL(clicked()),this,SLOT(hostGame()));
    ui.leftLayout->addWidget(sth);

    RoundButton* con = new RoundButton(QPixmap(":/eyecandy/connect.png"));
    con->setToolTip(tr("Connect to Game"));
    connect(con,SIGNAL(clicked()),this,SLOT(connectToGame()));
    ui.leftLayout->addWidget(con);

    RoundButton* start = new RoundButton(QPixmap(":/eyecandy/start.png"));
    start->setToolTip(tr("Start Game"));
    connect(start,SIGNAL(clicked()),this,SLOT(startGame()));
    ui.leftLayout->addWidget(start);

    RoundButton* endT = new RoundButton(QPixmap(":/eyecandy/endTurn.png"));
    endT->setToolTip(tr("End Turn"));
    connect(endT,SIGNAL(clicked()),this,SLOT(nextTurn()));
    ui.bottomLayout->addWidget(endT);
}

KonquestWindow::KonquestWindow(QWidget *parent)
    :QLabel(parent)
{
    ui.setupUi(this);
    this->setWindowFlags(Qt::WindowStaysOnTopHint);
    makeButtons();
    this->setMask(QPixmap(":/eyecandy/konquestMask.png").mask());
    this->setPixmap(QPixmap(":/eyecandy/konquestBg.png"));
    QGraphicsScene* scene = new QGraphicsScene(0.0f,0.0f,410.0f,410.0f);
    scene->setBackgroundBrush(QBrush(QPixmap(":/eyecandy/space.png")));
    ui.gameView->setScene(scene);
    gridVisable = true;
    infoDisplay = new PlanetInfoDisplay();
    infoDisplay->setZValue(1);
    scene->addItem(infoDisplay);
    infoDisplay->hide();
    makeGrid();

    colorList << Qt::red;
    colorList << Qt::blue;
    colorList << Qt::green;
    colorList << Qt::yellow;

    myPlayerId = 0;
    currentPlayerTurn = -10;

    gameSrcPlanet = -1;

    isHost = false;
    server = NULL;

    communicationLine = NULL;

    QDesktopWidget desktop;
    this->move(desktop.width()/2 - 300,desktop.height()/2 -300);

    chatLog = new ChatLogWidget();
    chatLog->hide();

    connect(chatLog,SIGNAL(messageTyped(QString)),this,
            SLOT(sendTextMsg(QString)));
    makeColorLabel();
}

KonquestWindow::~KonquestWindow()
{
    if(server)
        if(!server->isFinished())
            server->stopWorking();

    if(communicationLine)
        communicationLine->disconnectFromHost();

    delete communicationLine;
    delete server;
}
