#ifndef KONQUESTWINDOW_H
#define KONQUESTWINDOW_H

#include <QtGui/QWidget>
#include <QLabel>
#include <QBitmap>
#include <QMouseEvent>
#include <QGraphicsScene>
#include <QGraphicsLineItem>
#include <QVector>
#include <QMap>
#include <QDesktopWidget>
#include <QTcpSocket>
#include <QMessageBox>
#include <QKeyEvent>
#include <QInputDialog>
#include <QLineEdit>
#include <QPainter>
#include <QDateTime>
#include <cmath>
#include "ui_konquestwindow.h"
#include "roundbutton.h"
#include "planetitem.h"
#include "planetData.h"
#include "planetinfodisplay.h"
#include "chatlogwidget.h"
#include "serverthread.h"


class KonquestWindow : public QLabel
{
    Q_OBJECT

public:
    KonquestWindow(QWidget *parent = 0);
    ~KonquestWindow();

protected:
    void mousePressEvent(QMouseEvent* e);
    void mouseMoveEvent(QMouseEvent* e);
    void keyReleaseEvent(QKeyEvent* e);

private slots:
    void hideShowGrid();                                        //SLOT used for hidign and showing the gird
    void planetClicked(int id);                                 //a planet was clicked
    void planetTouched(int id,int x,int y);                     //planet touched display the information item
    void planetUntouched();                                     //hide the information item
    void performeClose();                                       //stop server thread disconnect sockets
    void hideShowChatLog();                                     //hide show the chat/log widget
    void sendTextMsg(QString msg);                              //setn a message form the chat/log widget

    //-----------------------Host-----------------------------------------------
    void hostGame();                                            //host a game
    void readServerAddress(QString address);                    //read the server address
    void startGame();                                           //start the game
    void newPlayerJoined(QTcpSocket* connection);               //a new player has join the game
    void massageFromPlayer();                                   //parse a massage comming form a player
    void sendPlayerList();                                      //broadcast the playerlist
    void sendPlanets();                                         //broadcast the planetstructs
    void serverListenError();

    //----------------------Guest-----------------------------------------------
    void connectToGame();                                       //connect to  a game
    void incommingMsg();                                        //parse a massage from the host

    //---------------------All--------------------------------------------------
    void nextTurn();                                            //end your turn
    void nextDay();                                             //day end fleets move production etc.
    bool makeMove(int scrPlanet,int dstPlanet,int shipCount);   //construct a fleet and add to the plane inbound fleets
    void disconnectionClose();                                  //connection broken
    void changeName();                                          //chang your name
    void makeColorLabel();                                      //make your color indycator
    void updateOwnPlanetShipCount();                            //draw ownPlanet ShipCount

private:
    void makeButtons();                                         //make buttons and widgets
    void makeGrid();                                            //make grid
    void makePlanets();                                         //create a random map host only
    Ui::KonquestWindow_ui ui;
    QVector<QGraphicsLineItem*> gridVector;                     //grid lines
    QList<QColor> colorList;                                    //player colors
    QMap<int,PlanetItem*> planetItems;                          //planet graphics items
    QMap<int,PlanetData> planetDataMap;                         //planet data structs
    int x,y;                                                    //windw pos
    bool gridVisable;                                           //is the grid shown
    PlanetInfoDisplay* infoDisplay;                             //info graphics item
    ChatLogWidget* chatLog;                                     //chat log widget
    bool isHost;                                                //true if hosting a game
    ServerThread* server;                                       //listening thread form host only
    QList<QTcpSocket*> connectedPlayers;                        //player list host only
    QMap<int,QString> playerNames;                              //map of player names for planet naming
    QTcpSocket* communicationLine;                              //line to server guest only
    QString nick;                                               //your player nick
    int myPlayerId;                                             //your player id
    int currentPlayerTurn;                                      //current player turn
    int dayCounter;                                             //day conut host only
    int gameSrcPlanet;                                          //source planet your turn only else -1
    QLabel* tLabel;                                             //red/green indicator green mens your turn
    QLineEdit* ipEdit;                                          //the conneciton ip edit line guest only
    QLineEdit* nameEdit;                                        //a line edit form chaniging your player name
    QLabel* colorLabel;                                         //player color indicator
};

#endif // KONQUESTWINDOW_H
