#ifndef PLANETDATA_H
#define PLANETDATA_H

#include <QString>
#include <QList>

struct Fleet
{
    Fleet()
    {
        shipCount = 0;
        ownerId = -1;
        roundsToArrival = 0;
    }

    Fleet(int shipCount,int ownerId,int roundsToArrival)
    {
        this->shipCount = shipCount;
        this->ownerId = ownerId;
        this->roundsToArrival = roundsToArrival;
    }

    Fleet(const Fleet& fleet)
    {
        this->shipCount = fleet.shipCount;
        this->ownerId = fleet.ownerId;
        this->roundsToArrival = fleet.roundsToArrival;
    }

    int shipCount;              //Fleet ship count
    int ownerId;                //Fleet ownerId (if != planet ownerId atacking)
    int roundsToArrival;        //rounds to arrival (if 0 fleet arrived take action)
};

struct PlanetPos
{
    PlanetPos()
    {
        x = 0;
        y = 0;
    }

    PlanetPos(int x,int y)
    {
        this->x = x;
        this->y = y;
    }

    PlanetPos(const PlanetPos& p)
    {
        this->x = p.x;
        this->y = p.y;
    }

    bool operator ==(const PlanetPos& p) const
    {
        if(this->x == p.x)
            if(this->y == p.y)
                return true;
        return false;
    }

    bool operator !=(const PlanetPos& p) const
    {
        return !this->operator ==(p);
    }

    bool operator<(const PlanetPos& p) const
    {
        if(this->x < p.x)
            return true;
        return false;
    }

    int x;
    int y;
};

struct PlanetData
{
    PlanetData()
    {
        this->planetName = "none";
        this->production = 0;
        this->shipCount = 0;
        this->ownerId = -1;
        this->defensBonus = 0.0f;
        this->pos = PlanetPos(0,0);
    }

    PlanetData(QString planetName,int production,int shipCount,int ownerId,
               float defensBonus,const PlanetPos& pos)
    {
        this->planetName = planetName;
        this->production = production;
        this->shipCount = shipCount;
        this->ownerId = ownerId;
        this->defensBonus = defensBonus;
        this->pos = pos;
    }

    PlanetData(const PlanetData& data)
    {
        this->planetName = data.planetName;
        this->production = data.production;
        this->shipCount = data.shipCount;
        this->ownerId = data.ownerId;
        this->defensBonus = data.defensBonus;
        this->inboundFleets = data.inboundFleets;
        this->pos = data.pos;
        this->inboundFleets = data.inboundFleets;
    }


    QString planetName;         //Planet name for displaing only

    int production;             //Planet ship production rate
                                //if pirates production = production * 0.25f
    int shipCount;              //Current planet ship count
    int ownerId;                //owner Id

    float defensBonus;          //Planet defens bonus multiplayer (atackersKilled = defenderCount + defenderCount/2 * defensBonus)
                                //if pirates defens bonus 0.2f
    PlanetPos pos;

    QList<Fleet> inboundFleets; //list of inbound fleets

    void addFleet(const Fleet& fleet)
    {
        inboundFleets.append(fleet);
    }
};

#endif // PLANETDATA_H
