#include "planetitem.h"

QRectF PlanetItem::boundingRect() const
{
    return QRectF(0.0f,0.0f,40.0f,40.0f);
}

void PlanetItem::paint(QPainter* painter,const QStyleOptionGraphicsItem*,QWidget*)
{
    painter->drawPixmap(0,0,myImage);
    painter->setPen(ownerColor);
    if(marked)
        painter->setBrush(QBrush(QColor(110,110,110,190)));
    else
        painter->setBrush(QBrush(Qt::transparent));
    painter->drawRect(1,1,38,38);
    if(showCount)
    {
        painter->setFont(QFont("Courier",8));
        painter->setPen(Qt::black);
        painter->setBrush(QBrush(QColor(10,10,10,190)));
        painter->drawRect(2,26,36,11);
        painter->setPen(Qt::white);
        QString shipCountS = tr("%1").arg(shipCount);
        if(shipCountS.length() > 4)
        {
            shipCountS.resize(3);
            shipCountS += "..";
        }
        painter->drawText(QRectF(2.0f,26.0f,36.0f,11.0f),tr("%1").arg(shipCountS)
                          ,QTextOption(Qt::AlignHCenter|Qt::AlignCenter));
    }
}

void PlanetItem::PlanetItem::mark()
{
    marked = true;
    update();
}

void PlanetItem::PlanetItem::unMark()
{
    marked = false;
    update();
}

void PlanetItem::showShipCount()
{
    showCount = true;
    update();
}

void PlanetItem::hideShipCount()
{
    showCount = false;
    update();
}

void PlanetItem::setShipCount(int count)
{
    shipCount = count;
}

void PlanetItem::mousePressEvent(QGraphicsSceneMouseEvent*)
{
    emit clicked(myId);
}

void PlanetItem::hoverEnterEvent(QGraphicsSceneHoverEvent* event)
{
    if(event->modifiers() == Qt::ShiftModifier)
    {
        emit touched(myId,pos().x(),pos().y());
        clearFocus();
    }
    else
        setFocus();
}

void PlanetItem::hoverLeaveEvent(QGraphicsSceneHoverEvent*)
{
    emit unTouched();
    clearFocus();
}

void PlanetItem::keyPressEvent(QKeyEvent* event)
{
    if(event->key() == Qt::Key_Shift)
    {
        emit touched(myId,pos().x(),pos().y());
        clearFocus();
    }
}

void PlanetItem::setOwnerColor(QColor color)
{
    ownerColor  = color;
    update();
}

PlanetItem::PlanetItem(QPixmap image,int id)
    :QObject(),QGraphicsItem()
{
    myImage = image;
    myId = id;
    shipCount = 0;
    showCount = false;
    ownerColor = QColor(Qt::transparent);
    this->setFlag(QGraphicsItem::ItemIsFocusable);
    this->setAcceptHoverEvents(true);
    marked = false;
}
